/************* FORWARD PROPAGATION CODE ***************/


int propnbrs(adjlist G, int v, settype s1, settype s2, settype s3, int depth)
/*
	v has a 1 color so is colored so delete
	that colorign from the availibility set of all neighbors 
	in which the color occurs.

	If avail of a nbr goes to zero, then return NO
	else RETURN YES. 
*/
{
	int w;
	w = FirstNbr(G,v);
	while (w != EndList(G->nbrlist[v]) ) {
		if (avail[v] & avail[w] ) {
			if (!RemoveAvail(avail[v],w,s1,s2,s3,depth,PROPONE))
				return NO;
		}
		w = NextNbr(G,v,w);
	}
	/* reduce graph immediately since v no longer relevant */
	Deletev(G,v,depth);
	return YES;
} /* end propnbrs */


#ifdef PROPTWO
int propdblnbrs(	adjlist G,
			int v, /*vertex with |avail[v]|= 2 */
			settype s1, 
			settype s2, /* sets fo avails 1 and 2 */
			settype s3,
			int depth) /* depth of search for stack recording */
/*
	Vertex v has avail two. Check all nbrs and add
	relevant tuples to the disjunct pairs sets.
	Return NO if any AddTuple fails due to contradiction
	Add vertex to the avail 3 set for triangle check.

	NOTE: if both DISJUNCTPAIR and PROPTHREE are turned off,
	then this routine may still check the common nbrs if
	cx == cy for some pair. See AddTuple
*/
{
	int w;
	int c;
	int i,j;

	if (!Member(G->vlist,v))  /* no longer in graph */
		return YES;

	if (numavail[avail[v]] != 2) 
		return YES; /* deal with this on s1 */

	w = FirstNbr(G,v);
	while (w != EndList(G->nbrlist[v]) ) {
		if   (numavail[avail[w]] == 2) {
		  c = avail[v] & avail[w];
		  if (c != 0)  {
			for(j=0;j<targetcolor;j++) {
				i = 1<< j;
				if (i&c) {
					if (!AddTuple(G,s1,s2,s3,
						v,w,(avail[v]&~i),
					        (avail[w]&~i),depth,PROPTWO))
						return NO;
					/* quit if v becomes forced */
					if (numavail[avail[v]] != 2)
                				return YES;
					/* next w if w becomes forced */
					if (numavail[avail[w]] != 2)
                				break;
				}
			}
		   }
		}
		w = NextNbr(G,v,w);
	}
	
#ifdef PROPTHREE
	if (!Member(s3,v))
		AddOne(s3,v); /* check for triangle formation */
#endif /* PROPTHREE */
	return YES;

} /* end propdblnbrs */
#endif /* PROPTWO */


int ApplyColor(	adjlist G,
		int v, /* vertex */
		int c, /* color */
		int depth) /*depth of search mark */
/*
	apply color c (in 2^k form) to vertex v and 
	propagate the coloring as far as it can be forced
	Must also stack the changes to avail sets
	and to vertices colored at this depth to
	allow unstacking on backtrack

	if coloring runs into contradiction then return NO
	else return  YES
*/
{
	settype s1; /* set for avail = 1 */
	settype s2; /* set for avail = 2 */
	settype s3; /* set for avail = 3 (triangle check) */

	int flag, success;

	int w;


	applynodes++; /* record stats */

	PushAvail(v, depth); 
	
	avail[v] = c;


	/* creating these each time is expensive? */
	s1 = MakeEmpty(order);
	AddOne(s1,v);
	totalnumcolored++;


	s2 = MakeEmpty(order);

// #define WANT3SKIP 1 
// this is likely not useful with the improved reduce
// ReduceNbrs at the beginning. Tests confirm that on 3-coloring of little
// impact - possibly negative due to time
#ifdef WANT3SKIP
	if (targetcolor == 3)
	{
		// for three coloring put all but the one
		// colored vertex on the s3 list to check for n4c's (Hajos)

		s3 = MakeCopy(G->vlist,order);
		Delete(s3,v);
	} else 
#endif
	{
		s3 = MakeEmpty(order);
	}

#ifdef DISJUNCTPAIR
	/* empty the pending queue */
	pendingfront = 0;
	pendingend = 0;

	/* set the active stack pointer to previous depth */
	disjtopdepth[depth] = disjtopdepth[depth-1];
#endif /* DISJUNCTPAIR */
	

	flag = YES;
	success = YES;
	while (flag) {
		
		if (EndList(s1) != (w = DeleteFirst(s1) ) ) {
			if ( ! propnbrs(G,w,s1,s2,s3,depth) ) {
				flag = NO;
				success = NO;
			}
		}

#ifdef PROPTWO
#ifdef DISJUNCTPAIR
		else if (pendingend > pendingfront) {
			if (! DisjProp(G,s1,s2,s3,depth)) {
				flag = NO;
				success = NO;
			}
		}
#endif  /* DISJUNCTPAIR */
		else if (EndList(s2) != (w = DeleteFirst(s2) ) ) {
			if ( ! propdblnbrs(G, w,s1,s2,s3,depth) ) {
				flag =  NO;
				success = NO;
			}
		} 
#ifdef PROPTHREE
		else if (EndList(s3) != (w = DeleteFirst(s3) ) ) {
			if ( ! triangle(G,w,s1,s2,s3,depth) ) {
				flag =  NO;
				success = NO;
			}
		}
#endif  /* PROPTHREE */
#endif  /* PROPTWO */
		else flag = NO;

	}

	Remove(s1);
	Remove(s2);
	Remove(s3);
/*
	if (! success) {
		selectValue[v] += d/10;
	}
*/

	return success;

} /* end ApplyColor */

/*******************END FORWARD PROPAGATION CODE **********/
