
/****************** INTEGER SET OPERATIONS OVER UNIVERSE 0..setlimit-1 ********
	Only works for sets of int
	Operations:
		MakeEmpty(setlimit) -- create and return an empty set 
				- over the universe 0..setlimit-1
		MakeCopy(old,setlimit) -- create and return a new copy
				of old
		Remove(S) --   free memory deleting set
		AddOne(S,w)  --  add one element w to the set
				- this implementation w will be the first
					element of the list supporting set
		Delete(S,w) -- delete the element w from S.
		DeleteFirst(S) --  return some element or return NONE if empty
				the element is deleted from S
			- this implementation takes first encountered
		First(S) - return first of list if not empty else S->NONE
		Next(S,w) - return next element following w in set S;
			- return first if w is s->NONE
			- return s->NONE if w is last element
			- error if w not in set -- return s->NONE.
		EndList(S) - returns S->NONE -- use e.g
				while (EndList(S) != (w = Next(S,w)) ) {...

		Setsize(S) - yields number of elements in S
		Member(S,w) - yields YES if w in S, NO else

	NONE is an integer set = to setlimit on MakeEmpty
	Three arrays are formed : 
		inset  -- binary indicates membership(1) or non-member(0)
		 prev, next -- doubly linked list of elements present

	The last (NONE) element is a dummy that is always present.
	
 *****************************************************/

#ifdef GLOBALIZE
/* when globalization is used we assume every set is of size order of graph */
#define NONE order
typedef struct _settype{
	short int size;	/* number of elements in set */
	char *inset; /* boolean to indicate if element is in the set */
	short int *next; /* next elemnt in set */
	short int *prev; /* previous element in the set */
} *settype;
#else
typedef struct _settype{
	short int size;	/* number of elements in set */
	char *inset; /* boolean to indicate if element is in the set */
	short int *next; /* next elemnt in set */
	short int *prev; /* previous element in the set */
	short int NONE; /* indicator not in this set */
} *settype;
#endif

/* inline functions USE after debugging */
#ifdef INLINE
#define Setsize(s) ((int) s->size) 
#else
int Setsize(settype s) 
{
	return ((int) s->size);
} /* end Setsize */
#endif

#ifdef INLINE
#define Member(s,w)  ((int)  s->inset[w] ) 
#else
int Member(settype s, int w) 
{
	if ( ( w < 0) || (w >= s->NONE) ){
		printf("Member: %d out of range[0..%d]\n",w,s->NONE);
		exit(1);
	}
	return((int) s->inset[w]);
} /* end Member */
#endif

settype MakeEmpty(int setlimit)
{
	settype s;

	nummakeempty++; /* track usage */

	if (NULL == (s = (settype) malloc(sizeof(struct _settype)) )){
		printf("MakeEmpty memory? cannot make set\n");
		exit(1);
	}
	if (NULL == (s->inset = (char *) calloc((unsigned) 
		setlimit+1, sizeof(int)) )){
		printf("MakeEmpty memory? cannot make set\n");
		exit(1);
	}
	if (NULL == (s->next = (short *) calloc((unsigned) 
		setlimit+1, sizeof(int)) )){
		printf("MakeEmpty memory? cannot make set\n");
		exit(1);
	}
	if (NULL == (s->prev = (short *) calloc((unsigned) 
		setlimit+1, sizeof(int)) )){
		printf("MakeEmpty memory? cannot make set\n");
		exit(1);
	}

	s->size = 0;

#ifdef GLOBALIZE
	s->next[NONE] = NONE;
	s->prev[NONE] = NONE;
	s->inset[NONE] = 1;
#else
	s->NONE = (short) setlimit;
	s->next[s->NONE]=s->NONE;
	s->prev[s->NONE]=s->NONE;
	s->inset[s->NONE] = 1;
#endif

	return s;
} /* end MakeEmpty */
	
void Remove(settype s)
{
	numremove++; /* usage tracking */
	free(s->next);
	free(s->prev);
	free(s->inset);
	free(s);
} /* end Remove */

void Delete(settype s, int elem)
{
#ifndef INLINE
#ifndef GLOBALIZE

	if ( ( elem <0 ) || ((short) elem > s->NONE) ) {
		printf("Element %d out of range[0..%d] in call to Delete\n",
			elem,s->NONE);
		return;
	}
#endif
#endif

#ifdef GLOBALIZE
	if (elem == (short) NONE) return;
#else
	if (elem == (short) s->NONE) return; /* nothing to delete */
#endif
	
        if (! s->inset[(short) elem] ) {
		return;
        }

	/* relink neighbors */
	s->next[s->prev[(short) elem]] = s->next[(short) elem];
	s->prev[s->next[(short) elem]] = s->prev[(short) elem];

	/* mark deleted */
	s->inset[(short) elem] = NO;
	s->size--;
} /* end Delete */

void AddOne(settype s, int elem)
{
#ifndef INLINE
#ifndef GLOBALIZE
	if ( ( elem <0 ) || ((short) elem >= s->NONE) ) {
		printf("Element %d out of range[0..%d] in call to AddOne\n",
			elem,s->NONE-1);
		return;
	}
#endif
#endif
	if (s->inset[(short) elem]) {
		printf("Set addition error: %d already in set\n",elem);
		return;
	}

	/* additions done at the front of doubly linked list */
#ifndef GLOBALIZE
	s->next[(short) elem] = s->next[s->NONE];
	s->prev[(short) elem] = s->NONE;
	s->prev[s->next[s->NONE]] = (short) elem;
	s->next[s->NONE] = (short) elem;
#else
	s->next[(short) elem] = s->next[NONE];
	s->prev[(short) elem] = NONE;
	s->prev[s->next[NONE]] = (short) elem;
	s->next[NONE] = (short) elem;
#endif
	

	s->inset[(short) elem] = YES;
	s->size++;
} /* end AddOne */
	
int DeleteFirst(settype s)
{
	short int first;
	/* take the first after s->NONE - may be s->NONE if empty */
#ifndef GLOBALIZE
	first = s->next[s->NONE];
#else
	first = s->next[NONE];
#endif

	/* delete on retrieval */
	Delete(s,(int) first);
	
	return ((int) first);
} /* end GetOne */

#ifdef INLINE
#ifdef GLOBALIZE
#define First(s) ((int) s->next[NONE])
#else
#define First(s) ((int) s->next[s->NONE] )
#endif /* GLOBALIZE */
#else /* NOT INLINE */
#ifdef GLOBALIZE
int First(settype s)
{
	return ((int) s->next[NONE]);
} /* end First */
#else
int First(settype s)
{
	return ((int) s->next[s->NONE]);
} /* end First */
#endif
#endif

int Next(settype s, int w)
{
#ifndef INLINE
#ifndef GLOBALIZE
	if ( ( w <0 ) || ((short) w > s->NONE) ) {
		printf("Element %d out of range[0..%d] in call to Next\n",
			w,s->NONE);
		return ((int) s->NONE);
	}
#endif
#endif
	if  (! s->inset[(short) w]) {
		printf("no element %d in the set\n",w);
#ifdef GLOBALIZE
		return NONE;
#else
		return ((int) s->NONE);
#endif
	}
	return( ( int) s->next[w] );
} /* end Next */

#ifdef INLINE
#ifdef GLOBALIZE
#define EndList(s) NONE
#else
#define EndList(s) ((int) s->NONE)
#endif /* GLOBALIZE */
#else
#ifdef GLOBALIZE
int EndList(settype s)
{
	return ((int) NONE);
} /* EndList */
#else
int EndList(settype s)
{
	return ((int) s->NONE);
} /* EndList */
#endif
#endif

settype MakeCopy(settype old, int setlimit)
{
	settype s;
	int w;

	s = MakeEmpty(setlimit);
	w = First(old);
	while (w != EndList(old) ) {
		AddOne(s,w);
		w = Next(old,w);
	}
	return s;
} /* end MakeCopy */

		
/********** End of set of integers over universe of 0..setlimit-1 ***/
		
