/************ REDUCE NEIGHBORS *************************/
int CheckMerge(adjlist G, int v, int depth, settype s1)
// check to see if there is a merge possible to vertex v
// works only for three coloring as of now.
// this added may29/99 J.C.
{
	int w,x,y,z;

	w = FirstNbr(G,v);
	while (w != EndList(G->nbrlist[v]) ) {
		x = NextNbr(G,v,w);
		while (x != EndList(G->nbrlist[v]) ) {
		   if (IsNbr(G,x,w) ) {
			// v,w,x forms a triangle - look for another vertex
			// adjacent to both x and w
			z = FirstNbr(G,w);
			while (z != EndList(G->nbrlist[w] )) {
				if ( (z != v) && (z != x) && IsNbr(G,z,x) )
				{
				    if (IsNbr(G,v,z)) 
				    { 
					return NO; // 4-clique
				    } else {
#ifdef TRACE
					printf("Mergev %d %d\n",v+1,z+1);
#endif
					Mergev(G,v,z,depth);
					avail[v] = avail[v]&avail[z];
					// When merged, put v back on s1 so it
					// will be checked again and return
					AddOne(s1,v);
					y = FirstNbr(G,v);
					while ( y != EndList(G->nbrlist[v]) ){
					   if (!Member(s1,y)) {
						AddOne(s1,y);
					   }
					   y = NextNbr(G,v,y);
					}
						
					return YES; 
				    }
				}
				// if not merged try next
				z = NextNbr(G,w,z);
			}
		   }
		   x = NextNbr(G,v,x);
		}
		w = NextNbr(G,v,w);
	}
	return YES;
}


int ReduceNbrs(adjlist G, int depth)
// return NO if CheckMerge detects not 3-colorable
// for k>=4 alwaays returns YES
{
	int v,w;
	settype s1;

	int availUnion;

	int readd,vdeleted,modified;

	s1 = MakeCopy(G->vlist,order);


	readd = 0;
	vdeleted=0;
	modified = 0;
	
	while (Setsize(s1) > 0) {
		v = DeleteFirst(s1);
		if (!Member(G->vlist,v)) continue;

		/*
			first we delete any edges that connect
			v to a neighbor with a disjoint avail set
			and form the union of the avail sets of
			the neighbors of v
		*/
		availUnion = 0;
		w = FirstNbr(G,v);
		while ( w != EndList(G->nbrlist[v]) ) {
			availUnion |= avail[w];
			if (0 == numavail[avail[w]&avail[v]] ) {
				modified++;
#ifdef TRACE
				printf("deleting edge %d %d\n",v+1,w+1);
#endif
				DeleteEdge(G,v,w,depth);
				/* make sure we test w now since its degree
					has changed */
				if ( ! Member(s1,w) ) {
					readd++;
					AddOne(s1,w);
				}
				/* nextnbr won't work since w no longer a nbr
					of v */
				w = FirstNbr(G,v);
			} else 
				w = NextNbr(G,v,w);
		}

		/* 
			Now delete v if it either has excess availability over
			the number of neighbors, or if it has some
			color available to it not available to any of
			its neighbors.
		*/
		if (  (numavail[ avail[v]] > Degree(G,v) )
		    || (numavail[ avail[v] & (~availUnion) ] > 0 )  ) 
		{
#ifdef TRACE
				printf("deleting vertex %d ",v+1);
				PrintAvail(v,targetcolor);
				printf(" deg = %d\n",Degree(G,v));
#endif
			/* delete all remaining edges */
			while ( Degree(G,v) > 0 ) {
				w = FirstNbr(G,v);
#ifdef TRACE
				printf("deleting edge %d %d\n",v+1,w+1);
#endif
				DeleteEdge(G,v,w,depth);
				/* make sure we test w now since its degree
					has changed */
				if (! Member(s1,w) ) {
					readd++;
					AddOne(s1,w);
				}
			}


			vdeleted++;
			Deletev(G,v,depth);
		} else if (targetcolor == 3)
		{
			if (CheckMerge(G,v,depth,s1)==NO)
				return NO;;
		}
		// PrintLocalGraph(G,targetcolor);
	}

	Remove(s1);
#ifdef TRACE
	printf("re-added = %d numdeleted = %d modified = %d\n",
		readd, vdeleted,modified);
#endif
	return YES;
}
/**************** END REDUCE NEIGHBORS ********/
