
/************************** SEARCH CONTROL ****************************/
/* interrupt processes */
int stopflag = 0; /* for interrrupt control */

void cpulimig()
{
        struct rlimit rlp;

        printf("CPU TIME LIMIT EXCEEDED -- let me clean up\n");
        fflush(stdout);
        stopflag++;
	if (stopflag > 1) {
		printf("Clean up time exceeded -- immediate halt\n");
		exit(1);
	}
        getrlimit(RLIMIT_CPU,&rlp);
	printf("current limit = %d\n",(int) rlp.rlim_cur);
        rlp.rlim_cur += 15; // allow 15 seconds max to clean up
        if ( setrlimit(RLIMIT_CPU, &rlp)) {
             printf("setrlimit failed\n");
#ifndef linux
            printf("errno = %d\n",errno);
#endif
            exit(1);
        }

} /* end cpulimig */

void intfcig()
{
        printf("INTERRUPT DETECTED -- let me clean up\n");
        fflush(stdout);
        stopflag++;
	if (stopflag >1) {
		printf("SECOND TIME - IMMEDIATE EXIT\n");
		exit(1);
	}
} /* end intfcid */


void InitSelect(adjlist G)
{
	int num3, num4, num5;
	int w,x,y,z;
	int *distq;

	/* multyipliers to increase selection of low avail vertices */
	/* for graphs I am testing preliminary, seems that should not
	   bias too heavy to less than 3 (for 4-colorable)
	*/
	mult[0] = 0.0;
	mult[1] = 4.0; /* forced should already be handled */
	mult[2] = 3.3; /* tuned for 4-coloring problems */
	mult[3] = 1.2;
	mult[4] = 1.0; 
	mult[5] = 0.0;
	mult[6] = 0.0;
	mult[7] = 0.0;

	/* to compute selectValue, we add all degrees to of
	   x and nbrs of x, plus bonuses for triangles, quads, an
	   5 cycles. 

	  NOTE that selectValue is modified later by search
	*/
	/* this is expensive but only done once */
	
	num3 = 0;
	x = Firstv(G);
	while (x != EndList(G->v) ) {
		/* initialize the distance queue to zero */
		if ( NULL == (distq = (int *) calloc(order,sizeof(int)))) {
			printf("Cannot allocate distq in startsearch\n");
			exit(1);
		}

		selectValue[x] = Degree(G,x);
		num3 = 0;
		num4 = 0;
		num5 = 0;

		y = FirstNbr(G,x);
		while (y != EndList(G->nbrlist[x]) ) {
			distq[y] = 1;
			selectValue[x] += Degree(G,y);
			z = FirstNbr(G,y);
			while (z != EndList(G->nbrlist[y]) ) {
			   if (z != x) {
				if (distq[z] == 1) num3++;
				else if (distq[z] == 2) num4++;
				else distq[z] = 2;
				w = FirstNbr(G,z);
				while (w != EndList(G->nbrlist[z])) {
					if (distq[w] == 2) num5++;
					w = NextNbr(G,z,w);
				}
			   } /* end if z != x */
			   z = NextNbr(G,y,z);
			}
			y = NextNbr(G,x,y);
		}
		free(distq);
		selectValue[x] += num3*1000 + num4*50 + num5*1;
#ifdef SELECTIONTRACE
		printf("x=%d num3=%d num4=%d num5=%d ",
			x+1,num3,num4,num5);
		printf("value=%d\n",selectValue[x]);
#endif
		x = Nextv(G,x);
	}

} /* end InitSelect */

int ForcedColor( adjlist G, int v, int a)
// june1,1999
// Return the number of vertices which willl be forced to a color if color a is
// assigned to vertex x
{
	int w;

	int fc;

	fc = 0;

	// Compute neighbors that would be forced 
	// If we do not exclude edges in the next part, we may not need this?
	w = FirstNbr(G,v);
	while (w != EndList(G->nbrlist[v])) {
		if (numavail[avail[w]&~a] == 1) fc++;
		w = NextNbr(G,v,w);
	}

	// compute forcings by disjuncts
	w = First(adjunct[v]);
	while(w != EndList(adjunct[v]) ) {
	   if ((Member(G->vlist,w)) && (!IsNbr(G,v,w))) {
		if ((disjmatrix[v][w].cx[0] != 0) 
			&& (disjmatrix[v][w].cx[0] != a)) fc++;
		if ((disjmatrix[v][w].cx[0] != 0) 
			&& (disjmatrix[v][w].cx[1] != a)) fc++;
	   }
	   w =  Next(adjunct[v],w);
	}
	return fc;
} /* end ForcedColor */

int MinForced(adjlist G, int v, int *c)
// return the minimum number of forced coloring for any color on v
// set a to the minimum color
{
	int i,a,min,j;
	min = order+1;

	for(i=0;i<targetcolor;i++)
	{
		a=1<<i;
		if (a & avail[v]) {
			j = ForcedColor(G,v,a);
			if (j<min) {
				min = j;
				(*c) = a;
			}
		}
	}
	return min;
}

int Select(adjlist G, int depth)
/* return a vertex of maximum value */
{
	int best, bestv;
	int v, currentvalue;
	int k;

	int c;


	if (Gorder(G) == 0) return -1;

	best =-1;

	if (depth > 1 /* MAGIC NUMBER */ ) {
		v = Firstv(G);
		bestv = v;
		k=1;

		while( v != EndList(G->vlist) ) {
			currentvalue = 
				MinForced(G,v,&c);
			if (currentvalue > best) {
				best = currentvalue;
				bestv = v;
			}
			else if (currentvalue == best) {
				if (Degree(G,v) > Degree(G,bestv))
				{
					bestv = v;
				} 
			}
			v = Nextv(G,v);
		}
		// record stats 
		selectedvalues[depth] += best;
		selecteddegrees[depth] += Degree(G,bestv);
		numvisits[depth]++;
	} else {
		v = Firstv(G);
		bestv = v;
		k=1;
		while( v != EndList(G->vlist) ) {
			/* current heuristic looks for least avail */
			if (numavail[avail[v]] < numavail[avail[bestv]] ) {
				best = selectValue[v];
				bestv = v;
				k = 1;
			} else 
			 if (numavail[avail[v]] == numavail[avail[bestv]] ){
			 	currentvalue = (int) 
				  (selectValue[v]*mult[numavail[avail[v]]]);
				if (currentvalue > best*1.1) {
					best = currentvalue;
					bestv = v;
					k = 1;
				}
				/* randomly choose between vertices 
				   of equal value*/
				else if (currentvalue >= best) {
					k++;
					if (0 == (random() % k) ) {
						bestv = v;
						best = currentvalue;
					}
				}
			}
			v = Nextv(G,v);
		}
	} /* else - depth <=7 */
	return bestv;
} /* end Select */


void RestoreAll( adjlist G, int depth)
{
	/* since RestoreAvail is called when at backtrack points
	   we can use this time to reset the maximu colored stat */
	if (maxcolor1000 < totalnumcolored) maxcolor1000 = totalnumcolored;

	/* Restore Availability sets */
	while (astack.depth[astack.top] == depth ) {
		/* restore color count for stats purposes */
		if ( 1 == numavail[avail[astack.vertex[astack.top]]] )
			totalnumcolored--;
		/* restore availability */
		avail[astack.vertex[astack.top]] = astack.avail[astack.top];
		astack.top--;
	}

	/* and we check the minimum colored stats here */
	if (mincolor1000> totalnumcolored) mincolor1000 = totalnumcolored;
	
	/* reconstruct graph */
	UndoAll(G,depth); 

#ifdef DISJUNCTPAIR
	/* Restore the disjunct pairs structure */
	UndoDisj(depth);
#endif
} /* end RestoreAll */
	


int search(adjlist G, int depth, int targetcolor)
/*
	Do recursive backtrack search with forward propagation
	and recursive graph reduction
*/
{
	int v,c,cindex;

	int j,k;

	int temp, minfwdval;

	int selectColor[SMALLLIMIT];


	searchnodes++;


	if (0 == (searchnodes % 20000)) {
		printstats(depth,Gorder(G));
	}

	/* record greatest depth out of every 1000*/
	if (depth > maxdepth1000) maxdepth1000 = depth; 

	if (-1 == (v = Select(G,depth) )) { 
		return YES;	
	}

	minfwdval = order;

#ifdef SEARCHTRACE
	printf("select avail[%d] = %d numavail= %d\n",
		v+1,avail[v],numavail[avail[v]]);
#endif

	for(j=0;j<targetcolor;j++) {
		selectColor[j] = (1<<j);
	}
	/* permute here for randomization */
	for(j=0;j<targetcolor-1;j++) {
		k = j + (random() % (targetcolor-j));
		temp = selectColor[j];
		selectColor[j] = selectColor[k];
		selectColor[k] = temp;
	}
	// select smallest impact color and swap it to the front.
	j = MinForced(G,v,&c);
	for (j=0;j<targetcolor;j++)
	{
		if (c==selectColor[j]) {
			selectColor[j] = selectColor[0];
			selectColor[0] = c;
			break;
		}
	}

#ifdef SEARCHTRACE
	printf("color order ");
	for(j=0;j<targetcolor;j++) printf(" %d",selectColor[j]);
	printf("\n");
#endif

	cindex = 0;
	while ( (cindex < targetcolor ) && (!stopflag) ) {
	    c = selectColor[cindex];
	    if ( c & avail[v]) {
		temp = totalnumcolored;
#ifdef SEARCHTRACE
		printf("Try v=%d c=%d depth=%d\n",v+1,c,depth);
#endif
		if (ApplyColor(G, v,c,depth)) {

			if (minfwdval > (totalnumcolored - temp) )
				minfwdval = totalnumcolored - temp;
			
/*			TOO EXPENSIVE AND LITTLE IMPACT(??)
			if (ReduceNbrs(G,depth)) verify works with 
			new possible fail!
*/
			if (search(G,depth+1,targetcolor) )  {
				return YES; 
			}
			/* record furtherest backtrack point every 1000 nodes*/
			if (depth < mindepth1000) mindepth1000 = depth;

	 	} else { /* if fail ApplyColor */
			if (minfwdval > (totalnumcolored - temp) )
				minfwdval = totalnumcolored - temp;
		}

	     	RestoreAll(G,depth); /* undoes coloring restoring 
				all avail sets modified at this depth 
				and undoes graph changes at this depth */
	   } /* c in avail */
	   cindex++; /* check nextcolor */
		
	} /* all colors checked */
	
	// if (minfwdval < order)
	// 	selectValue[v] += minfwdval;

		

	return NO; /* this coloring branch failed. */

} /* end search */

int StartSearch(adjlist G, int targetcolor)
/*
	a place to initialize the selectValue array and
	choose and color the first small clique to avoid
	needless backtrack
*/

{
	int a,b,c; /* up to three vertices selected initially */
	int r,s;	/* temporary vertex selectors */
	int triflag;
	int k;
	int depth;

	depth =1;
	if (targetcolor ==2) { /* two coloring is easy */
		while (EndList(G->vlist) != (a = Firstv(G)) ) {
			if (!ApplyColor(G,a,1,depth)) 
				return NO;
			depth++;
		}
		return YES;
	} /* end two coloring */
		
	/* preprune the graph */
	if (!ReduceNbrs(G,depth))
	{
		printf("ReduceNbr detected 4-clique on 3-coloring\n");
		return NO;
	}

	if (Setsize(G->vlist) == 0)
	{ // reduction has shown colorable
		printf("Colored by reduction prepruning\n");
		return YES;
	}


	InitSelect(G);
	
	/*
		Now we select the maximum value vertex, color it 1
		and its best nbr and color it 2 
		NOTE: randomization on vertices of equal value
	*/

	// Sanity Check:
	// Possible error (or reduction not complete) -- if ever occurs
	// should be checked out (I don't think it can occur).
	if (Setsize(G->vlist) <= targetcolor)
	{
		printf("ERROR (?) in StartSearch: ");
		printf("incomplete collapse in Reduction\n");
	} 
	if (Setsize(G->vlist) < 3)
	{
		printf("ERROR (?) less than 3 vertices after reduction\n");
		printf("Skipping pre-color step in StartSearch\n");
		depth = 1;
	} else {
		printf("Graph size before initial selects %d\n",
			Setsize(G->vlist));
		printf("\n------------------------\n\n");
		a = 0;
		k = 1;
		for (r=1;r<order;r++) {
			if (selectValue[r] > selectValue[a]) {
				a=r;
				k=1;
			}
			else if (selectValue[r] == selectValue[a]) {
				k++;
			  	if  ( 0 == (random() % k)) a=r;
			}
		}
	
		r = FirstNbr(G,a);
		b = r;
		triflag = 0; /* set true when a triangle found, and c is set */
		c=0;
		while( r  != EndList(G->nbrlist[a]) ) {
			s = FirstNbr(G,r);
			while (s != EndList(G->nbrlist[r]) ) {
				if (IsNbr(G,a,s)) {
					if (!triflag)  {
						triflag = 1;
						b = r;
						c = s;
					} else if (selectValue[r] + selectValue[s] >
					      selectValue[b] + selectValue[c]) {
						b = r;	
						c = s;
					}
				} 
				else if ( (!triflag) 
				  && (selectValue[r] > selectValue[b]) ) {
					b = r;
				}
				s = NextNbr(G,r,s);
			}
			r = NextNbr(G,a,r);
		}
	
	
		printf("Initial vertices selected are a= %d b = %d",a+1,b+1);
		if(triflag)
			printf(" and c = %d",c+1);
		printf("\n");
	
		if (cheatflag) { /* assumes we have a (partial) coloring */
			printf("CHEATING IN PROGRESS .. cheatdepth = %d\n",cheatdepth);
			if (! ApplyColor(G,a,cheatclr[a],1) ) {
				printf("WHAT THE HEY? Cannot color first vertex\n");
				return NO;
			}
			
			if ( Member(G->vlist,b)) /* may be colored */
			    if (! ApplyColor(G,b,cheatclr[b],2) ) {
				printf("WHOA! Cannot color second vertex\n");
				return NO;
			}
			if (triflag) {
				if (Member(G->vlist,c))/* may be colored */
			    	if (! ApplyColor(G,c,cheatclr[c],3) ) {
					printf("OOPS! Cannot color third vertex\n");
					return NO;
				}
				depth = 4;
			} else
				depth  = 3;
			
			/* complete the cheating to the limit of knowledge */
			while (depth <= cheatdepth ) {
				a = Select(G,depth);
				if (!ApplyColor(G,a,cheatclr[a],depth)) {
				 printf("CHEATFAIL depth =%d, a= %d, cheatclr = %d\n",
					depth, a+1, cheatclr[a]);
				 return NO;
				}
				depth++;
			}
	
		} else { /* standard no knowledge coloring */
			if (! ApplyColor(G,a,1,1) ) {
				printf("WHAT THE HEY? Cannot color first vertex\n");
				return NO;
			}
			
			if ( Member(G->vlist,b))  /* may be colored */
			    if (! ApplyColor(G,b,2,2) ) {
				printf("WHOA! Cannot color second vertex\n");
				return NO;
			}
			if ((triflag) && ( Member(G->vlist,c))) /* may be colored */ 
			{
			    	if (! ApplyColor(G,c,4,3) ) {
					printf("OOPS! Cannot color third vertex\n");
					return NO;
				}
				depth = 4;
			} else
				depth  = 3;
		} /* end of standard no knowledge or cheat */
		depth =1;
	}
	
	printf("Graph Size before search is %d\n",Setsize(G->vlist));

	/* complete the coloring */
	if (search(G,depth,targetcolor) ) return YES;
	return NO;

} /* end startsearch */

/******************* END SEARCH CONTROL ***************************/
