
/*
        Title: Smallk Source File.
        file: smallk.c
        does: smallk routine
        Source: Joseph Culberson's Coloring Page
                http://web.cs.ualberta.ca/~joe/Coloring/index.html
        Author: Joseph Culberson
        email: joe@cs.ualberta.ca

NOT FOR DISTRIBUTION AT THIS TIME

	Copyright (c) 1997 Joseph Culberson. All rights reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:
	1. Redistributions of source code must retain the above copyright
   	notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
   	notice, this list of conditions and the following disclaimer in the
   	documentation and/or other materials provided with the distribution.
	3. All advertising materials mentioning features or use of this 
	software must display the following acknowledgement:
     	This product includes software developed by J. Culberson at the
     	University of Alberta, Edmonton.
	4. Neither the name of the University nor the names of its contributors
   	may be used to endorse or promote products derived from this software
   	without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND ANY
	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
	PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	THIS SOFTWARE IS SUPPLIED WITHOUT ANY SUPPORT SERVICES. 

*/

#include "mysys.h"
#include "graph.h"
#include "colorrtns.h"
#include<malloc.h>
#include "smallk.h"


// #define DEBUG 
// #define TRITRACE
// #define SEARCHTRACE 
// #define DISJTRACE
// #define STACKTRACE
// #define TRACE
/* #define PRINTLOCALONCE  */

#define INLINE
#define GLOBALIZE


/* Set up for testing combinations of forward propagations */
#define PROPONE 0 
#define PROPTWO 1
#define PROPTHREE 2
#define DISJUNCTPAIR 3
#define DISJPROP 4

#define NUMTYPES 5

#define YES 1
#define NO 0


int	targetcolor; 	/* Global targetcolor since used everywhere */
int     *selectValue;   /*values to be used in setting selection priorities*/
double	mult[SMALLLIMIT]; /* hand coded numers to bias by numavail
                used in Select, set in InitSelect */


int	cheatflag,	/* indicates if cheating ot occur */
	cheatdepth,	/* how many cheat colors to use */
	*cheatclr;	/* cheat colors in binary form - must be input
			   as numbers in range 1..k for vertices 0..n-1
			  NOTE does not use the cheat set in graph since
			  may wish to use a different deceptive coloring
			*/

#include"stats.c" 	/* program statistics */
#include"integerset.c"	/* sets of vertices in this program */
#include"availset.c"	/* avail sets of vertices - boolean vectors */
#include"localgraph.c"	/* operations and maintenance of the local copy of
				graph including stacking changes to the graph*/
#include"stacks.c"	/* stacks for recording and undoing avail proagation*/
#include"disjunct.c"	/* disjunctive propagation structures */
#include"triangle.c"	/* checking 3-color forcings on triangles */
#include"forward.c"	/* for proagating avail restricitions and identity
				merges */
#include"reduce.c" 	/* reduction of the graph */
#include"search.c"	/*recursive search and select routines */
#include"restore.c"	/* after successful search, use the graph stack 
				to restore and color the vertices */

/************** INITIAL STARTUP *******************/

int smallk(popmembertype *m, int tclrs)
{
	

	int i,j;
	int flag;
	int color[MAXVERTEX];	/* colors assigned to vertices 
				   for return to caller purposes */

	adjlist G;		/* the local copy of the graph */
	
int	vperm[MAXVERTEX];	/* stores vertex permutation */


	for(i=0;i <order; i++) vperm[i] = m->vc[i].vertex;

	targetcolor = tclrs;
	printf("Target colors = %d\n",targetcolor);
	if ( ( targetcolor > SMALLLIMIT) || (targetcolor < 1) ) {
		printf("Target color %d out of range\n",targetcolor);
		exit(1);
	}

#ifndef WITHCHEAT
	cheatflag = 0;
#else
	printf("DO YOU WISH TO CHEAT (0==no, 1== yes) ");
	scanf("%d",&cheatflag);
	printf("%d\n",cheatflag);

	if (cheatflag == 1) {
		if (NULL == (cheatclr = (int *) calloc(order,sizeof(int)) )) {
			printf("Cannot allocate cheatclr\n");
			exit(1);
		}
		printf("Enter cheatdepth ");
		scanf("%d",&cheatdepth);
		printf("%d\n",cheatdepth);

		/* we do not echo colors*/
		printf("Enter the colors for all vertices\n");
		for(i=0;i<order;i++)  {
			scanf("%d",&j);
			printf("%3d ",j);
			cheatclr[i] = (1 << (j-1));
		}
		printf("\n");

	} else if (cheatflag != 0) {
		printf("Illegal cheatflag\n");
		exit(1);
	}

#endif
	/* initialize availabilities of colors */
	initavail(targetcolor);
	setcount();


	if (NULL == (selectValue = (int *) calloc(order, sizeof(int)) ) ) {
                printf("cannot allocate selectValue\n");
                exit(1);
        }

	/* make local copy of graph using adjacency list variant */
	G = MakeLocalGraph(vperm);

	/* initial the avail stack for restoring avails during backtrack */
	InitAvailStack(targetcolor);

#ifdef PRINTLOCALONCE
	printf("Initially\n");
	PrintLocalGraph(G,targetcolor);
#endif


	InitStats();

#ifdef DISJUNCTPAIR
	DisjAlloc();
#endif

	/* do the search */
	flag = StartSearch(G,targetcolor);

	printf("SEARCH COMPLETES\n");
#ifdef PRINTLOCALONCE
	PrintLocalGraph(G,targetcolor);
	PrintChangeStack(G);
#endif

	
	printstats(0,Gorder(G));

	if (!flag) {
		printf("Coloring failed to find %d-coloring\n",
			targetcolor);
		if (stopflag >0) 
			printf("INTERRUPTED SEARCH\n");
                DeleteGraph(G);
#ifdef DISJUNCTPAIR
                DisjRelease();
#endif
                free(selectValue);
                RemoveAvailStack();
                RemoveStats();

		return NO; /* no coloring completed */
	}

	/* to complete the coloring, we have to traverse the change stack */
	RestoreAndColor(G,targetcolor);

	/* if coloring complete translate into general system form */
	for(i=0;i<order;i++) {
		j=0;
		while (  ( j<targetcolor) 
			&&( ! (avail[i] & (1<<j) )) ) j++;
		color[i] = j+1;
	}



	/* copy the coloring for return to the system */
	for(i=0;i<order;i++) {
		m->vc[i].vertex = i;
		m->vc[i].color = color[i]; /* temporary! */
	}

        DeleteGraph(G);
#ifdef DISJUNCTPAIR
        DisjRelease();
#endif
        free(selectValue);
        RemoveAvailStack();
        RemoveStats();
	return YES;
} /* end smallk */


/* END OF FILE smallk.c */
