/*
	RANDOMIZED BACKTRACK FOR SMALL K-COLORABLE GRAPHS
*/

/*
        Title: smallk Setup and Call Source File.
        file: smallmain.c
        does: hcmain routine
        Source: Joseph Culberson's Coloring Page
                http://web.cs.ualberta.ca/~joe/Coloring/index.html
        Author: Joseph Culberson
        email: joe@cs.ualberta.ca

NOT FOR DISTRIBUTION AT THIS TIME

	Copyright (c) 1997 Joseph Culberson. All rights reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:
	1. Redistributions of source code must retain the above copyright
   	notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
   	notice, this list of conditions and the following disclaimer in the
   	documentation and/or other materials provided with the distribution.
	3. All advertising materials mentioning features or use of this 
	software must display the following acknowledgement:
     	This product includes software developed by J. Culberson at the
     	University of Alberta, Edmonton.
	4. Neither the name of the University nor the names of its contributors
   	may be used to endorse or promote products derived from this software
   	without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND ANY
	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
	PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	THIS SOFTWARE IS SUPPLIED WITHOUT ANY SUPPORT SERVICES. 

*/

#include "mysys.h"
#include "smallk.h"
#include "graph.h"
#include "colorrtns.h"

/* Global Time info */
long sseconds,smicrosecs;
long useconds,umicrosecs;
double user_t,system_t, total_t;
struct rusage tmp;

/* for linux on the dual processor, we must add cpu and sys times
	as the break down by linux is nonsense
*/
void inittime()
{
	long int i;
	i = getrusage(RUSAGE_SELF,&tmp);
	useconds = tmp.ru_utime.tv_sec;
	umicrosecs = tmp.ru_utime.tv_usec;
	sseconds = tmp.ru_stime.tv_sec;
	smicrosecs = tmp.ru_stime.tv_usec;
	user_t = system_t = total_t = (double) 0.0;
}

void elapsedtime() /* from the last inittime */
{
	long i;
	i = getrusage(RUSAGE_SELF,&tmp);
	user_t = (double) (tmp.ru_utime.tv_sec - useconds)  +
		((double) (tmp.ru_utime.tv_usec - umicrosecs)/1000000.0);
	system_t = (double) (tmp.ru_stime.tv_sec - sseconds)  +
		((double) (tmp.ru_stime.tv_usec - smicrosecs)/1000000.0);
	total_t = system_t + user_t;
}

void colorsearch(char *name, int clrs)
/* 
	This version uses the greedy, this routine 
	asks for the various parameters.
*/

{
	popmembertype m;
	int i;

	char info[256];       

        int pid;
        /* process id */
        pid = getpid();
        printf("Process pid = %d\n",pid);



	/* initialize vertex ordering to decreasing degree */
	for(i=0;i<order;i++) m.vc[i].vertex = i;

	computedeg();
	qsort((char *) m.vc,(int) order,
              sizeof(struct vrtxandclr), (compfunc)decdeg);

	if (smallk(&m,clrs)) {

		getcolorinfo(&m);
	
		printinfo(&m);

		verifycolor(&m);

		elapsedtime();

        	sprintf(info, "SMALLK cpu = %5.2f pid = %d", total_t, pid);

        	fileres(name, &m,info);

	}


	/* printcoloring(&m); */
	

}

int main(int argc, char *argv[])
{
	int seed;
        struct rlimit rlp;

	if (argc < 4) {
		printf("\tUsage: smallk file seed colors [cpulimit]\n");
		exit(1);
	}

	about("SMALL K");

        if (argc > 4 ) {
                getrlimit(RLIMIT_CPU,&rlp);
                rlp.rlim_cur =atoi(argv[4]);
#ifndef linux
                printf("Cpulimit set to %d\n", rlp.rlim_cur);
#else
                printf("Cpulimit set to %ld\n", rlp.rlim_cur);
#endif
                if ( setrlimit(RLIMIT_CPU, &rlp)) {
                        printf("setrlimit failed\n");
#ifndef linux
                        printf("errno = %d\n",errno);
#endif
                        exit(1);
                }
                signal(SIGXCPU,cpulimig);
        }

        signal(SIGINT,intfcig);


	/* Set up graph */
	inittime();
	getgraph(argv[1]);
	elapsedtime();
	printf("GRAPH SETUP user = %5.2f sys = %5.2f total = %5.2f\n",
		user_t, system_t, total_t);

	/* perform color search */

	/* Random initialization for search */
	seed = atoi(argv[2]);
	printf("Randomization seed =  ");
	printf(" %d\n",seed);
	srandom(seed);

	inittime();
	colorsearch(argv[1], atoi(argv[3]));
	elapsedtime();
	printf("Coloring time user = %5.2f sys = %5.2f total = %5.2f\n",
		user_t, system_t, total_t);
		

	return(0);
}

