
/********************* COLLECT EXECUTION STATS FOR ANALYSIS ***********/
/*
	Note that these are all global variables that set here and
	there to record various aspects of program performance
	Some stats are reset in PrintStats to record differences
	as the search proceeds and so Printstats takes an argument "depth".
*/

int	searchnodes =0,		/* number of search nodes reached */
	applynodes = 0,		/* number of calls to apply colors */
	maxdepth1000 = 0,
	alltimemaxdepth = 0,
	mindepth1000 = MAXVERTEX, /* record the search range every ? nodes */
	furthestback = MAXVERTEX;	/* furthest back from beginning */

	// heuristic Tracking 
int	*selectedvalues; /* total seleccted values at each depth */
int	*selecteddegrees; /* total seleccted degrees at each depth */
int	*numvisits; /* number of visit to depth  */


int 	**numremoved;	/* track colors removed by various operations
			at each different depth to maxdepth1000 */

int	**numfailed; 	/* number of detected violations by each type
			operation at each depth */

int	**numforced;    /* number of vertices with forced colors 
			by operation type and depth */

int	dblmerge = 0,		/* number of merges induced by propdbl */
	availdfsfail = 0,	/* color failure detected by availdfs */
				/* the last two should be equal */
	checkfivefail = 0;	/* number of failures forced by checkfive */

int	fulltri = 0,		/* all vertices of tri have a common nbr */
	trimerge = 0;		/* merges induced by triangle */

int 	maxcolor1000 = 0,	/* maximum number of nodes assigned a color
				   during a 1000(?) nodes of search */
	mincolor1000 = 0, 	/* minimum over last reporting interval nodes */
	alltimemax = 0,		/* maximum number of nodes ever colored */
	totalnumcolored = 0;	/* number colored at any time, kept only to
				   set maxcolor1000, alltimemax */

int	mingraphsize =999999, 	/* minimum grah size since last print stats*/
	maxgraphsize = 0,	/* maximum graph size since last print */
	glblmingraph =999999;	/* overall maximum graph size */

int	edgesremoved =0,	/* keep tally of edges removed */
	maxedgesremoved = 0,	/* since last stats */
	minedgesremoved = 0; 	/* since last stats */


/* memory leak tracking */
int 	nummakeempty = 0, 
	numremove  = 0;	/* set allocation deallocation balance */



void printstats(int depth, int graphsize)
{
	int i,j;
	
	printf("\nS010 Current Depth %d\n",depth);
	printf("S020 Search nodes %d\n", searchnodes);
	printf("S030 Calls to apply colors %d\n",applynodes);
	printf("S040 Max. depth since last stats %d\n",maxdepth1000);
	printf("S050 Min. depth since last stats %d\n",mindepth1000);
	if (furthestback > mindepth1000) furthestback = mindepth1000;
	printf("S051 Over all Min Depth %d\n",furthestback);
	
	
	printf("...Global...\n");
	if (alltimemax < maxcolor1000) 
		alltimemax = maxcolor1000;
	printf("S180 Maximum nodes colored: since start %d\n",alltimemax);
	printf("S190 Maximum nodes colored: since last stats %d\n",maxcolor1000);
	printf("S200 Minimum colored nodes since last stats %d\n", mincolor1000);
	if (glblmingraph > mingraphsize) 
		glblmingraph = mingraphsize;
	printf("S210 Minimum graph size since start %d\n",glblmingraph);
	printf("S220 Maximum graph size since last stats %d\n",maxgraphsize);
	printf("S230 Minimum graph size since last stats %d\n",mingraphsize);
	printf("S240 Maximum edges removed since last stats %d\n",
				maxedgesremoved);
	printf("S250 Minimum edges removed since last stats %d\n",
				minedgesremoved);
	
	printf("...Memory usage...\n");
	printf("Makeempty create = %d remove = %d diff = %d\n",
			nummakeempty,numremove, nummakeempty-numremove);
	printf("\t-------------\n");

	if (alltimemaxdepth < maxdepth1000) alltimemaxdepth = maxdepth1000;

//	printf("\t\tAverage\n");
//	printf("Depth\t Values \t Degrees\n");
//	for(i=1;i<alltimemaxdepth;i++) {
//		if (numvisits[i]>0)
//		printf("%5d\t%8.3f\t%8.3f\n",i,
//			((double) selectedvalues[i])/ ((double) numvisits[i]),
//			((double) selecteddegrees[i])/ ((double) numvisits[i]));
//	}

	printf("...NUMBER FORCED COLORS...\n");
	printf("Depth\tType\n");
	printf("     \t");
	for(j=0;j<NUMTYPES;j++) printf("%7d\t",j);
	printf("\n----------------------\n");
	
	for(i=1;i<alltimemaxdepth;i++) {
		printf("%3d",i);
		for(j=0;j<NUMTYPES;j++) {
			printf("\t%7d",numforced[j][i]);
		}
		printf("\n");
	}

	printf("...NUMBER COLORS REMOVED...\n");
	printf("Depth\tType\n");
	printf("     \t");
	for(j=0;j<NUMTYPES;j++) printf("%d\t",j);
	printf("\n");
	
	for(i=1;i<alltimemaxdepth;i++) {
		printf("%3d",i);
		for(j=0;j<NUMTYPES;j++) {
			printf("\t%7d",numremoved[j][i]);
		}
		printf("\n");
	}
		
	printf("...NUMBER FAILED...\n");
	printf("Depth\tType\n");
	printf("     \t");
	for(j=0;j<NUMTYPES;j++) printf("%d\t",j);
	printf("\n");
	
	for(i=1;i<alltimemaxdepth;i++) {
		printf("%3d",i);
		for(j=0;j<NUMTYPES;j++) {
			printf("\t%7d",numfailed[j][i]);
		}
		printf("\n");
	}
		
		

	fflush(stdout);

	/* reset search depth limits */
	maxdepth1000 = depth;
	mindepth1000 = depth;

	/* reset for next stats */
	maxcolor1000 = totalnumcolored;
	mincolor1000 = totalnumcolored;

	mingraphsize = graphsize;
	maxgraphsize = graphsize;
	maxedgesremoved = edgesremoved;
	minedgesremoved = edgesremoved;
}

void InitStats()
{
	int i;
	if( NULL == (selectedvalues = (int *) calloc(order, sizeof(int)))) {
		printf("calloc Fail InitStats.1\n");
		exit(1);
	}
	if( NULL == (selecteddegrees = (int *) calloc(order, sizeof(int)))) {
		printf("calloc Fail InitStats.1\n");
		exit(1);
	}
	if( NULL == (numvisits = (int *) calloc(order, sizeof(int)))) {
		printf("calloc Fail InitStats.1\n");
		exit(1);
	}

	if( NULL == (numremoved = (int **) calloc(NUMTYPES, sizeof(int *)))) {
		printf("calloc Fail InitStats.1\n");
		exit(1);
	}
	if( NULL == (numforced = (int **) calloc(NUMTYPES, sizeof(int *)))) {
		printf("calloc Fail InitStats.2\n");
		exit(1);
	}
	if( NULL == (numfailed = (int **) calloc(NUMTYPES, sizeof(int *)))) {
		printf("calloc Fail InitStats.3\n");
		exit(1);
	}

	for(i=0;i<NUMTYPES;i++) {
	 if (NULL == (numremoved[i] = (int *) calloc(order,sizeof(int)))) {
		printf("callof fail InitStats-4\n");
		exit(1);
	 }
	 if (NULL == (numforced[i] = (int *) calloc(order,sizeof(int)))) {
		printf("callof fail InitStats-5\n");
		exit(1);
	 }
	 if (NULL == (numfailed[i] = (int *) calloc(order,sizeof(int)))) {
		printf("callof fail InitStats-6\n");
		exit(1);
	 }
	}
	
}/* end InitStats */
void RemoveStats()
{
        int i;
        for(i=0;i<NUMTYPES;i++) {
                free(numremoved[i]);
                free(numforced[i]);
                free(numfailed[i]);
        }

        free(numforced);
        free(numfailed);
        free(numremoved);
} /* end RemoveStats */
/***************** END STATS COLLECTION *******************************/
